@@ -3,7 +3,7 @@
 # documentation at end of file
 
 use strict;
-use Getopt::Long;
+use Getopt::Long qw(:config no_ignore_case bundling);
 use Pod::Usage;
 use Statistics::Lite qw(sum min max mean median stddevp);
 use Statistics::Descriptive;
@@ -33,7 +33,7 @@ my $PARAMETRIC    = 1;
 my $ORDINAL       = 0;
 	
 use constant LOG10 => log(10);
-my $VERSION = '1.33';
+my $VERSION = '1.63';
 
 print "\n This program will correlate positions of occupancy between two datasets\n\n";
 
@@ -63,7 +63,6 @@ my (
 	$position,
 	$set_strand,
 	$norm_method,
-	$interpolate,
 	$gz,
 	$cpu,
 	$help,
@@ -72,24 +71,23 @@ my (
 
 # Command line options
 GetOptions( 
-	'in=s'        => \$infile, # the input data file
-	'out=s'       => \$outfile, # name of output file 
-	'db=s'        => \$database, # name of database
-	'ddb=s'       => \$data_database, # database containing datasets
-	'ref=s'       => \$refDataSet, # reference dataset
-	'test=s'      => \$testDataSet, # test dataset
-	'pval!'       => \$find_pvalue, # calculate student t-test
-	'shift!'      => \$find_shift, # calculate optimum shift
-	'radius=i'    => \$radius, # for collecting data when shifting
-	'pos=s'       => \$position, # set the relative feature position
+	'i|in=s'        => \$infile, # the input data file
+	'o|out=s'       => \$outfile, # name of output file 
+	'd|db=s'        => \$database, # name of database
+	'D|ddb=s'       => \$data_database, # database containing datasets
+	'r|ref=s'       => \$refDataSet, # reference dataset
+	't|test=s'      => \$testDataSet, # test dataset
+	'pval!'         => \$find_pvalue, # calculate student t-test
+	'shift!'        => \$find_shift, # calculate optimum shift
+	'radius=i'      => \$radius, # for collecting data when shifting
+	'p|pos=s'       => \$position, # set the relative feature position
 	'force_strand|set_strand'  => \$set_strand, # enforce an artificial strand
 				# force_strand is preferred option, but respect the old option
-	'norm=s'      => \$norm_method, # method of normalization
-	'interpolate!' => \$interpolate, # interpolate the position data
-	'gz!'         => \$gz, # compress output
-	'cpu=i'       => \$cpu, # number of execution threads
-	'help'        => \$help, # request help
-	'version'     => \$print_version, # print the version
+	'norm=s'        => \$norm_method, # method of normalization
+	'z|gz!'         => \$gz, # compress output
+	'c|cpu=i'       => \$cpu, # number of execution threads
+	'h|help'        => \$help, # request help
+	'v|version'     => \$print_version, # print the version
 ) or die " unrecognized option(s)!! please refer to the help documentation\n\n";
 
 # Print help
@@ -153,7 +151,7 @@ validate_or_request_dataset();
 my $start_time = time;
 # check whether it is worth doing parallel execution
 if ($cpu > 1) {
-	while ($cpu > 1 and $Data->last_row / $cpu < 1000) {
+	while ($cpu > 1 and $Data->last_row / $cpu < 100) {
 		# We need at least 1000 lines in each fork split to make 
 		# it worthwhile to do the split, otherwise, reduce the number of 
 		# splits to something more worthwhile
@@ -204,10 +202,10 @@ sub check_defaults {
 	if (defined $position) {
 		# check the position value
 		unless (
+			$position eq 'm' or
 			$position == 5 or
 			$position == 3 or
-			$position == 4 or
-			$position eq 'm'
+			$position == 4
 		) {
 			die " Unknown relative position '$position'!\n";
 		}
@@ -218,10 +216,6 @@ sub check_defaults {
 		$position = 4;
 	}
 
-	unless (defined $interpolate) {
-		$interpolate = 1;
-	}
-
 	if ($norm_method) {
 		unless ($norm_method =~ /^rank|sum$/i) {
 			die " Unrecognized normalization method! see help\n";
@@ -241,8 +235,7 @@ sub check_defaults {
 	}
 
 	if ($parallel) {
-		# conservatively enable 2 cores
-		$cpu ||= 2;
+		$cpu ||= 4;
 	}
 	else {
 			# disable cores
@@ -407,20 +400,6 @@ sub collect_correlations {
 	my $stream = $Data->row_stream;
 	while (my $row = $stream->next_row) {
 		
-		
-		# determine reference point
-		my $ref_point;
-		if ($position == 5) {
-			$ref_point = $row->strand >= 0 ? $row->start : $row->end;
-		}
-		elsif ($position == 4) {
-			$ref_point = int( ( ( $row->start + $row->end ) / 2 ) + 0.5);
-		}
-		elsif ($position == 3) {
-			$ref_point = $row->strand >= 0 ? $row->end : $row->start;
-		}
-		
-		
 		# Collect positioned data
 		my %ref_pos2data;
 		my %test_pos2data;
@@ -428,53 +407,38 @@ sub collect_correlations {
 			# collecting data in a radius around a reference point
 			
 			# collect data
-			%ref_pos2data = $row->get_position_scores(
-				'ddb'       => $ddb,
-				'dataset'   => $refDataSet,
-				'start'     => $ref_point - (2 * $radius),
-				'stop'      => $ref_point + (2 * $radius),
-				'position'  => $position, 
-				'value'     => 'score',
+			%ref_pos2data = $row->get_relative_point_position_scores(
+				ddb       => $ddb,
+				dataset   => $refDataSet,
+				position  => $position,
+				extend    => (2 * $radius),
 			);
-			%test_pos2data = $row->get_position_scores(
-				'ddb'       => $ddb,
-				'dataset'   => $testDataSet,
-				'start'     => $ref_point - (2 * $radius),
-				'stop'      => $ref_point + (2 * $radius),
-				'position'  => $position, 
-				'value'     => 'score',
+			%test_pos2data = $row->get_relative_point_position_scores(
+				ddb       => $ddb,
+				dataset   => $testDataSet,
+				position  => $position,
+				extend    => (2 * $radius),
 			);
 		}
 		else {
 			# just collect data over the region
 			# collect data
-			%ref_pos2data = $row->get_position_scores(
-				'ddb'       => $ddb,
-				'dataset'   => $refDataSet,
-				'position'  => $position, 
-				'value'     => 'score',
+			%ref_pos2data = $row->get_region_position_scores(
+				ddb       => $ddb,
+				dataset   => $refDataSet,
+				position  => $position,
 			);
-			%test_pos2data = $row->get_position_scores(
-				'ddb'       => $ddb,
-				'dataset'   => $testDataSet,
-				'position'  => $position, 
-				'value'     => 'score',
+			%test_pos2data = $row->get_region_position_scores(
+				ddb       => $ddb,
+				dataset   => $testDataSet,
+				position  => $position,
 			);
 		}
 		
 		# Verify minimum data count 
 		if (
-			(
-				not $interpolate and (
-					scalar(keys %ref_pos2data) < 5 or 
-					scalar(keys %test_pos2data) < 5
-					# 5 is an arbitrary minimum number
-				)
-			) or
-			(
 				sum( map {abs $_} values %ref_pos2data ) == 0 or 
 				sum( map {abs $_} values %test_pos2data ) == 0
-			)
 		) {
 			# not enough data points to work with
 			$row->value($r_i, '.');
@@ -495,36 +459,20 @@ sub collect_correlations {
 		}
 		
 		
-		# Interpolate data points
-		# this will improve calculating Pearson values 
-		# especially when data is sparse
-		if ($interpolate) {
-			# pass the data reference and the region length
-			interpolate_values(\%ref_pos2data, $row->length);
-			interpolate_values(\%test_pos2data, $row->length);
-		}
-		
-		
 		# Normalize the data
 		# there are couple ways to do this
 		if ($norm_method =~ /rank/i) {
 			# convert to rank values
 			# essentially a Spearman's rank correlation
 			normalize_values_by_rank(\%ref_pos2data);
-			normalize_values_by_rank(\%ref_pos2data);
+			normalize_values_by_rank(\%test_pos2data);
 		}
 		elsif ($norm_method =~ /sum/i) {
 			# normalize the sums of both
 			# good for read counts
 			normalize_values_by_sum(\%ref_pos2data);
-			normalize_values_by_sum(\%ref_pos2data);
+			normalize_values_by_sum(\%test_pos2data);
 		}
-# 		elsif ($norm_method =~ /median/i) {
-# 			# median scale the values to normalize
-# 			# just another way
-# 			normalize_values_by_median(\%ref_pos2data);
-# 			normalize_values_by_median(\%ref_pos2data);
-# 		}
 		
 		
 		# Collect the (normalized) data from the position data
@@ -592,7 +540,7 @@ sub collect_correlations {
 			# change strand
 			if ($set_strand) {
 				# user is imposing a strand
-				if ($$row->strand < 0) {
+				if ($row->strand < 0) {
 					# only change shift direction if reverse strand
 					$best_shift *= -1;
 				}
@@ -628,7 +576,6 @@ sub add_new_columns {
 	# extra information
 	$Data->metadata($r_i, 'data_database', $data_database) if $data_database;
 	$Data->metadata($r_i, 'normalization', $norm_method) if $norm_method;
-	$Data->metadata($r_i, 'interpolate', 1) if $interpolate;
 	if ($radius) {
 		$Data->metadata($r_i, 'radius', $radius);
 		$Data->metadata($r_i, 'reference_point', $position == 5 ? 
@@ -673,90 +620,6 @@ sub add_new_columns {
 }
 
 
-sub interpolate_values {
-	# the point of this is to interpolate scores at positions where no 
-	# actual score was collected
-	
-	# the actual size of the collected data region is determined by the 
-	# radius value, in other words whether we collected data in radius 
-	# around a reference point or just for the requested region
-	
-	# initiate
-	my $data = shift;
-	my $length = shift;
-	my $limit; # the length of the data requested
-	if ($radius) {
-		$limit = 2 * $radius; # hard encoded, that's what we're working with
-	}
-	else {
-		$limit = $length;
-	}
-	
-	# Fill out the data so that we have all elements filled
-	for (
-		my $i = $radius ? 0 - $limit : 1;
-		# the starting point depends on whether we collected in a 
-		# radius or just the region itself
-		$i <= $limit; 
-		$i++
-	) {
-		unless (exists $data->{$i}) {
-			# using default value of zero
-			$data->{$i} = 0;
-		}
-	}
-	
-	# Interpolate data
-	my $i = $radius ? 0 - $limit : 1; # starting point
-	while ($i <= $limit) {
-		# walk through the data
-		# we're not using a for loop here because we may interpolate
-		# more than one element at a time
-		
-		
-		# Look for zero values
-		if ($data->{$i} == 0) {
-			# we have a zero value
-			
-			# find the next non-zero value to interpolate
-			my $next = $i + 1;
-			while ($next < $limit) {
-				last if ($data->{$next} != 0); # we found it
-				$next++;
-			}
-			
-			# check if we reached the limit
-			if ($next == $limit) {
-				# that's it, nothing left to interpolate
-				# return what we have
-				return;
-			}
-			
-			# determine fractional number
-			# difference between next non-zero number and previous number
-			# divided by the number of elements in between
-			my $fraction = ($data->{$next} - $data->{$i - 1}) / 
-				(abs($next - $i) + 1);
-			
-			# apply fractions
-			for (my $n = $i; $n < $next; $n++) {
-				$data->{$n} = $data->{$i - 1} + ($fraction * (abs($n - $i) + 1));
-			}
-			
-			# reset for next round
-			$i = $next + 1;
-		}
-		
-		else {
-			# a non-zero number, no interpolation necessary, next please
-			$i++;
-		}
-	}
-	# done, just return
-}
-
-
-
 sub calculate_optimum_shift {
 	# Calculate an optimal shift that maximizes the Pearson correlation
 	
@@ -766,34 +629,40 @@ sub calculate_optimum_shift {
 	# we will calculate a Pearson correlation for each shift
 	# calculating window of 2*radius, shifting 2*radius across region
 	my %shift2pearson;
-	for (
-		my $shift = 0 - $radius; 
-		$shift <= $radius;
-		$shift++
-	) {
-		# shift is from -1radius to +1radius
-		# starting point is at -1radius to +1radius
-		# so adding together we get first window from -2radius to 0
-		# and last window is from 0 to +2radius
-		# this will be compared to the reference window, which does not move
+	for my $shift (1 .. $radius) {
+		# we will test shifts incrementally from 0 to radius in both directions
 		
-		# generate the array of test values
-		my @test_data; 
+		## forward shift
+		my @test_data;
 		for my $i ((0 - $radius + $shift) .. ($radius + $shift)) {
 			push @test_data, $test_pos2data->{$i} || 0;
 		}
-		
 		# check the test array
 		if ( min(@test_data) == max(@test_data) ) {
 			# no variance!
 			next;
 		}
-		
 		# calculate Pearson correlation
 		my $pearson = calculate_pearson($ref_data, \@test_data); 
-		if (defined $pearson and $pearson > 0.6) {
+		if (defined $pearson and $pearson > 0.5) {
 			$shift2pearson{$shift} = $pearson;
 		}
+		
+		## Reverse shift
+		@test_data = ();
+		for my $i ((0 - $radius - $shift) .. ($radius - $shift)) {
+			push @test_data, $test_pos2data->{$i} || 0;
+		}
+		# check the test array
+		if ( min(@test_data) == max(@test_data) ) {
+			# no variance!
+			next;
+		}
+		# calculate Pearson correlation
+		my $pearson = calculate_pearson($ref_data, \@test_data); 
+		if (defined $pearson and $pearson > 0.5) {
+			$shift2pearson{$shift * -1} = $pearson;
+		}
 	}
 	
 	# identify the best shift
@@ -802,7 +671,7 @@ sub calculate_optimum_shift {
 	foreach my $shift (keys %shift2pearson) {
 		if ($shift2pearson{$shift} == $best_r) {
 			# same as before
-			if ($best_shift != 1000000 and $shift < $best_shift) {
+			if ($best_shift != 1000000 and abs($shift) < abs($best_shift)) {
 				# if it's a lower shift, then keep it, 
 				# but not if it's original
 				$best_r = $shift2pearson{$shift};
@@ -977,29 +846,36 @@ A script to calculate correlations between two datasets along the length of a fe
 
 correlate_position_data.pl [--options] <filename>
   
-  Options:
-  --in <filename>
-  --out <filename> 
-  --db <name | filename>
-  --ddb <name | filename>
-  --ref <type | filename>
-  --test <type | filename>
-  --pval
-  --shift
-  --radius <integer>
-  --pos [5|m|3]                 (m)
-  --norm [rank|sum]
-  --force_strand
-  --(no)interpolate
-  --cpu                         (2)
-  --gz
-  --version
-  --help
+  Options for data files:
+  -i --in <filename>               input file: txt bed etc
+  -o --out <filename>              optional output file, default overwrite 
+  -d --db <name>                   alternate annotation database
+  
+  Options for data sources
+  -D --ddb <name|file>             data or BigWigSet database
+  -r --ref <dataset|filename>      reference data: bw, name, etc
+  -t --test <dataset|filename>     test data: bw, name, etc
+  
+  Options for correlating data
+  --pval                           calculate P-value by ANOVA
+  --shift                          determine optimal shift to match datasets
+  --radius <integer>               radius in bp around reference point to calculate
+  -p --pos [5|m|3]                 reference point to measure correlation (m)
+  --norm [rank|sum]                normalization method between datasets
+  --force_strand                   force an alternate strand
+  
+  General options:
+  -c --cpu <interger>              number of threads (4)
+  -z --gz                          compress output with gz
+  -v --version                     print version and exit
+  -h --help                        show extended documentation
 
 =head1 OPTIONS
 
 The command line flags and descriptions:
 
+=head2 Options for data files
+
 =over 4
 
 =item --in <filename>
@@ -1007,7 +883,7 @@ The command line flags and descriptions:
 Specify the input file of features. The features may be comprised of 
 name and type, or chromosome, start, and stop. Strand information is 
 optional, but is respected if included. A feature list may be 
-generated with the program B<get_features.pl>. The file may be 
+generated with the program L<get_features.pl>. The file may be 
 compressed with gzip.
 
 =item --out <filename>
@@ -1016,19 +892,24 @@ Specify the output filename. By default it rewrites the input file.
 
 =item --db <name | filename>
 
-Specify the name of a C<Bio::DB::SeqFeature::Store> annotation database 
+Specify the name of a L<Bio::DB::SeqFeature::Store> annotation database 
 from which gene or feature annotation may be derived. A database is 
 required for generating new data files with features. This option may 
 skipped when using coordinate information from an input file (e.g. BED 
 file), or when using an existing input file with the database indicated 
-in the metadata. For more information about using annotation databases, 
-see L<https://code.google.com/p/biotoolbox/wiki/WorkingWithDatabases>. 
+in the metadata. 
+
+=back
+
+=head2 Options for data sources
+
+=over 4
 
 =item --ddb <name | filename>
 
 If the data to be collected is from a second database that is separate 
 from the annotation database, provide the name of the data database here. 
-Typically, a second C<Bio::DB::SeqFeature::Store> or BigWigSet database 
+Typically, a second L<Bio::DB::SeqFeature::Store> or BigWigSet database 
 is provided here. 
 
 =item --ref <type | filename>
@@ -1040,11 +921,17 @@ correlate. These may be GFF type or name in a database or BigWigSet, or
 they may be a BigWig or even Bam file. Both options are required. If 
 not provided, they may be interactively chosen from the database.
 
+=back
+
+=head2 Options for correlating data
+
+=over 4
+
 =item --pval
 
 Perform an ANOVA analysis between the test and reference data sets and 
 report a P-value. By default, this performs a dependent, parametric 
-ANOVA. This requires the C<Statistic::ANOVA> module to be installed. 
+ANOVA. This requires the L<Statistic::ANOVA> module to be installed. 
 Please refer to the module documentation for details. If your needs 
 require a change to the test, you may edit the parameters at the top 
 of this script. For convenience, the P-values are reported as -Log10(P) 
@@ -1092,11 +979,11 @@ the direction of the reported shift. This requires the presence of a
 data column in the input file with strand information. The default is 
 no enforcement of strand.
 
-=item --(no)interpolate
+=back
 
-Interpolate missing or zero positioned values in each window for both 
-reference and test data. This will improve the Pearson correlation 
-values, especially for sparse data. Enabled by default.
+=head2 General options
+
+=over 4
 
 =item --gz
 
@@ -1125,26 +1012,32 @@ genomic position, indicating a change in nucleosome position.
 Two statistics may be calculated. First, it will calculate a a Pearson
 linear correlation coefficient (r value) between the datasets (default). 
 Additionally, an ANOVA analysis may be performed between the datasets and 
-generate a P-value.  
+generate a P-value. 
 
 By default, the correlation is determined between the data points 
 collected over the entire length of the feature. Alternatively, a 
 radius and reference point (default is midpoint) may be provided 
 that sets the window for collecting scores and calculating a correlation.
 
-To ensure a more reliable Pearson value, missing values or values of 
-zero are interpolated from neighboring values, when possible. Also, 
-values may be normalized using one of two methods. The values may be 
+In general, to ensure a more reliable Pearson value, fragment ChIP or 
+nucleosome coverage should be used rather than point (start or midpoint) 
+data, as it will give more reliable results. Fragment coverage is more 
+akin to smoothened data and gives better results than interpolated point 
+data. 
+
+Normalized read-depth data should be used when possible. If necessary, 
+Values can be normalized using one of two methods. The values may be 
 converted to rank positions (compare to Kendall's tau), or scaled such 
 that the absolute sum values are equal (for example, when working with 
 sequence tag read counts).
 
 In addition to calculating a correlation coefficient, an optimal shift 
 may also be calculated. This essentially shifts the data, 1 bp at a time, 
-in order to identify a shift that would produce a higher correlation. 
-The window is shifted from -2 radius to +2 radius relative to the 
-reference point, and the highest correlation is reported along with the 
-shift value that generated it.
+in order to identify a shift that would produce a higher correlation. In 
+other words, what amount of movement to the left or right would make the 
+test data look like the reference data? The window is shifted from -2 
+radius to +2 radius relative to the reference point, and the highest 
+correlation is reported along with the shift value that generated it. 
 
 =head1 AUTHOR
 
